@@ -1,26 +1,8 @@
 {-# OPTIONS -fglasgow-exts #-}
-{- arch-tag: Logger main definition
-Copyright (C) 2004-2009 John Goerzen <jgoerzen@complete.org>
-
-This program is free software; you can redistribute it and/or modify
-it under the terms of the GNU Lesser General Public License as published by
-the Free Software Foundation; either version 2.1 of the License, or
-(at your option) any later version.
-
-This program is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-GNU Lesser General Public License for more details.
-
-You should have received a copy of the GNU Lesser General Public License
-along with this program; if not, write to the Free Software
-Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
--}
-
 {- |
    Module     : System.Log.Logger
-   Copyright  : Copyright (C) 2004-2009 John Goerzen
-   License    : GNU LGPL, version 2.1 or above
+   Copyright  : Copyright (C) 2004-2011 John Goerzen
+   License    : BSD3
 
    Maintainer : John Goerzen <jgoerzen@complete.org> 
    Stability  : provisional
@@ -172,6 +154,7 @@ module System.Log.Logger(
                                -- make your job easier.
                                debugM, infoM, noticeM, warningM, errorM,
                                criticalM, alertM, emergencyM,
+                               removeAllHandlers,
                                traplogging,
                                -- ** Logging to a particular Logger by object
                                logL,
@@ -205,7 +188,7 @@ logger hierarchy. -}
                                updateGlobalLogger
                                ) where
 import System.Log
-import System.Log.Handler(LogHandler)
+import System.Log.Handler(LogHandler, close)
 import System.Log.Formatter(LogFormatter)
 import qualified System.Log.Handler(handle)
 import System.Log.Handler.Simple
@@ -470,6 +453,14 @@ updateGlobalLogger ln func =
     do l <- getLogger ln
        saveGlobalLogger (func l)
 
+-- | Allow gracefull shutdown. Release all opened files/handlers/etc.
+removeAllHandlers :: IO ()
+removeAllHandlers =
+    modifyMVar_ logTree $ \lt -> do
+        let allHandlers = Map.fold (\l r -> concat [r, handlers l]) [] lt
+        mapM_ (\(HandlerT h) -> close h) allHandlers
+        return $ Map.map (\l -> l {handlers = []}) lt
+
 {- | Traps exceptions that may occur, logging them, then passing them on.
 
 Takes a logger name, priority, leading description text (you can set it to
